# frozen_string_literal: true

module Hermes
  class UpdateSubscription
    include Hermes::Concerns::CommonFieldsConcern

    field :name, type: String
    field :object_id, type: BSON::ObjectId
    field :reference_model_name, type: String
    field :email, type: Boolean
    field :web, type: Boolean
    field :slack, type: Boolean
    field :sms, type: Boolean
    field :user_email, type: String
    
    writer_auth_roles 'superadministrateur'

    validates :name, uniqueness: { scope: [:object_id, :reference_model_name] }

    writer_roles 'Superadmin', 'Administrator'
    reader_roles 'Manager'
    owner_field :user_id

    common_indexes *%i[
      name
      object_id
      reference_model_name
      email
      web
      slack
      sms
    ]
    index(name: 1, object_id: 1, reference_model_name: 1)
    index(updated_at: 1, name: 1, email: 1)

    before_save :before_save_actions

    def before_save_actions
      self.user_email = User.find(object_id)&.email  if object_id_changed? && object_id && reference_model_name == 'User'
    end 

    def self.format_date(date)
      date.strftime('%d-%m-%Y-%H:%M:%S')
    end 

    def self.generate_reporting_CSV(options = {})
      starttime = options[:starttime] || DateTime.now - 7.days
      endtime = options[:endtime] || DateTime.now
      tagset_batch_id = options[:tagset_batch_id]
      responsible_id = options[:responsible_id]

      raise Exception, "Missing tagset_batch_id" unless tagset_batch_id
      raise Exception, "Missing responsible_id" unless responsible_id

      file_name = "from-#{format_date(starttime)}-to-#{format_date(endtime)}-#{HERMES_CONFIG['environment']}-reporting-notifications-offres-privileges.csv"
      file = CSV.open(file_name, 'ab', {:col_sep => ";"})
      file << [
        'entity_title',
        'entity_siret',
        'entity_id_cristal',
        'user_email',
        'updated_at',
      ]

      r = Hermes::UpdateSubscription.where(
        name: 'user.new_reserved_offer',
        updated_at: starttime..endtime,
        email: true,
      )

      r.batch_size(5).no_timeout.each_with_index do |us, i|
        begin
          user = User.find(us.object_id)
          hr = Hermes::HousingRequest.where(user_id: us.object_id).order_by(created_at: :desc).first
          entity = hr&.entity

          line = [
            entity&.title,
            entity&.siret,
            entity&.id_cristal,
            user&.email,
            us.updated_at,
          ]
          file << line

        rescue Exception => e
          line = [us.id, "Erreur de traitement: #{e}\n"]
          file << line
        end
      end
      file.close
      puts "CSV with #{r.count} lines generated"

      dcsv = Doc.create(responsible: responsible_id, tagset_ids: [tagset_batch_id], type: "file", title: file_name)
      dcsv.blob = File.read(file_name)

      if existing_doc = Doc.find_by(:file_md5 => dcsv.file_md5, :id.ne => dcsv.id, :version_of => nil)
        dcsv.destroy
        dcsv = existing_doc
        dcsv.update(title: file_name)
      else
        dcsv.save
      end

      dcsv
    end 

  end
end
